


import numpy
import matplotlib.pyplot as plt

pump_wavelength=1064*10**-9
stokes_wavelength=1555*10**-9
w_s=2*(numpy.pi)*3*10**8/(stokes_wavelength)      #angular stokes frequency
w_p=2*(numpy.pi)*3*10**8/(pump_wavelength)        #angular pump frequency
T_p=32/10**6                                    #mirror parameters at pump and stokes
T_s=38/10**6
R_p=1-(100+(T_p*10**6))/10**6
R_s=1-(100+(T_s*10**6))/10**6


mode_overlap=2*pump_wavelength/(pump_wavelength+stokes_wavelength)   #make this more accurate by accounting for cavity length
constant=.08318       #from Deniz's calculations

'''conversion efficiency is almost completely linear at low powers like this, 
so rather than trying to calculate all the matrix elements and stuff I'm just using an overall constant based on Deniz's calculations'''

def scaled_area(Wo,L):
    return(L*pump_wavelength/(4*numpy.arctan((L/(2*numpy.pi*Wo*Wo/pump_wavelength)))))


def cavity_mode(roc,L,wavelength):
     
    A=1+((2*L**2 - L*(4*roc))/(roc**2))
    B=2*L+((2*L**3)-3*L**2*(2*roc))/(2*roc**2)
    C=(4*L-2*roc)/(roc**2)
    D=1+(2*L**2-L*(4*roc))/(roc**2)
    if abs(A+D)>=2:
        #print('cavity not stable')
        return(False)
    else:
        return((wavelength/(2*numpy.pi))**.5 * (L*(2*roc-L))**.25)


def power(E,Wo,L):
    return((E**2)*.5*scaled_area(Wo,L)*(.002654))
    
def E_field(Power,Wo,L):
    return((Power*2*376.7/scaled_area(Wo,L))**.5)


def mixing(P_pump_i,pressure,L,roc):    #P_pump_i=incident pump power coupled to cavity (about 12W incident)
    Wo_p=cavity_mode(roc,L,pump_wavelength)
    Wo_s=cavity_mode(roc,L,stokes_wavelength)
    if Wo_s==False or Wo_p==False:
        print('cavity not stable')
    L_p=(-3*10**8/(2*L))*numpy.log(R_p)
    L_s=(-3*10**8/(2*L))*numpy.log(R_s)
    E_pump_i=E_field(P_pump_i,Wo_p,L)
    K=3*10**8/L * numpy.sqrt(T_p)*E_pump_i
    G= 1/16 * ((5.625*10**-13)*pressure)*3*10**8*.002654 * mode_overlap #constant is sqrt(e/u)
    E_pump_s=(L_s/G)**.5    
    E_stokes_s=(w_s*(K-L_p*E_pump_s)/(w_p*G*E_pump_s))**.5
    P_pump_s=power(E_pump_s,Wo_p,L)
    P_stokes_s=power(E_stokes_s,Wo_p,L)
    P_stokes_out=P_stokes_s*T_s
    P_pump_out=P_pump_s*T_p
    efficiency=(pressure*L)**2*constant*(P_stokes_out*P_pump_out*((.00022539/Wo_p)**4))
    if K<L_p*E_pump_s:
        #print('below threshold')
        return(0,P_pump_out,0)
    #print('Mixing Efficiency=     '+str(efficiency)+'\nTransmitted Pump Power=  '+str(1000*P_pump_out)+' mW'+'\nTransmitted Stokes Power='+str(1000*P_stokes_out)+' mW')
    return(efficiency, P_pump_out,P_stokes_out)


def frequency_overlap(L,laser_FWHM=None, cavity_FWHM=None, finesse=23798):
    FSR=(3*10**8)/(2*L)
    if laser_FWHM==None:
        return 1
    if cavity_FWHM==None:
        cavity_FWHM=FSR/finesse
    f_overlap=min(1,cavity_FWHM/laser_FWHM)  #don't want a value>1
    return(f_overlap)
        

def plot_efficiency(pressure):
    for roc in (1,.5,.3,.1):
        length=numpy.linspace(.05*roc,roc,900)
        plot_length=list()
        efficiency=list()        
        for L in length:
            if cavity_mode(roc,L,pump_wavelength)!=False:
                plot_length.append(L)                    
                efficiency.append(mixing(frequency_overlap(L,300000)*10,pressure,L,roc)[0])
        if efficiency != False:
            efficiency=[10**6*a for a in efficiency]
            plot_length=[100*a for a in plot_length]
                    
            plt.plot(plot_length, efficiency,label='RoC='+str(int(100*roc))+ ' cm')
            plt.legend()
    plt.xlabel('Cavity Length (cm)',fontsize=20)
    plt.ylabel('Mixing Efficiency *$10^{-6}$',fontsize=20)
    plt.title('Modulation Efficiency vs Cavity Length at '+str(pressure)+' atm',fonsize=20)   

def plot_stokes(L,roc,pressure):
    Wo_p=cavity_mode(roc,L,pump_wavelength)
    Wo_s=cavity_mode(roc,L,stokes_wavelength)
    P_stokes_s=list()
    P_pump_i=numpy.linspace(0,10,1000)
    L_p=(-3*10**8/(2*L))*numpy.log(R_p)
    L_s=(-3*10**8/(2*L))*numpy.log(R_s)
    for P in P_pump_i:
         E_pump_i=E_field(P,Wo_p,L)
         K=3*10**8/L * numpy.sqrt(T_p)*E_pump_i
         G= 1/16 * ((5.625*10**-13)*pressure)*3*10**8*.002654 * mode_overlap
         E_pump_s=(L_s/G)**.5  
         if K<L_p*E_pump_s:
            P_stokes_s.append(0)
         else:
             E_stokes_s=(w_s*(K-L_p*E_pump_s)/(w_p*G*E_pump_s))**.5
             P_stokes_s.append(power(E_stokes_s,Wo_p,L))
    P_stokes_out=[a*T_s for a in P_stokes_s]
    #print(P_stokes_out,P_pump_i)
    plt.plot(P_pump_i,P_stokes_out)
            
def plot_pump(L,roc,pressure):
    efficiency=list()
    pump=numpy.linspace(0,10,1000)
    for p in pump:
        efficiency.append(mixing(p,pressure,L,roc)[0])
        scaled_efficiency=[10**6*a for a in efficiency]
    plt.plot(pump,scaled_efficiency)
    plt.xlabel('Coupled Pump Power (W)',fontsize=20)
    plt.ylabel('Mixing Efficiency *$10^{-6}$',fontsize=20)
    plt.title('Modulation Efficiency vs Coupled Pump Power for L='+str(100*L)+' cm, RoC='+str(100*roc)+' cm, p='+str(pressure)+' atm',fontsize=20)

         
def plot_pressure(L,roc):
    efficiency=list()
    pressure=numpy.linspace(0,1,1000)
    for p in pressure:
        efficiency.append(mixing(10,p,L,roc)[0])
        scaled_efficiency=[10**6*a for a in efficiency]
    plt.plot(pressure,scaled_efficiency)
    plt.xlabel('Pressure (atm)',fontsize=20)
    plt.ylabel('Mixing Efficiency *$10^{-6}$',fontsize=20)
    plt.title('Modulation Efficiency vs Pressure for L='+str(100*L)+' cm, RoC='+str(100*roc)+' cm',fontsize=20)
    plt.tight_layout() 
